@@ -131,16 +131,16 @@ static void irq_submit (unsigned int irq, counter_t value)
 
 	vl.values = values;
 	vl.values_len = 1;
-	vl.time = time (NULL);
 	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
 	sstrncpy (vl.plugin, "irq", sizeof (vl.plugin));
+	sstrncpy (vl.type, "irq", sizeof (vl.type));
 
-	status = snprintf (vl.type_instance, sizeof (vl.type_instance),
+	status = ssnprintf (vl.type_instance, sizeof (vl.type_instance),
 			"%u", irq);
 	if ((status < 1) || ((unsigned int)status >= sizeof (vl.type_instance)))
 		return;
 
-	plugin_dispatch_values ("irq", &vl);
+	plugin_dispatch_values (&vl);
 } /* void irq_submit */
 
 static int irq_read (void)
@@ -151,8 +151,8 @@ static int irq_read (void)
 	FILE *fh;
 	char buffer[BUFSIZE];
 	unsigned int irq;
-	unsigned int irq_value;
-	long value;
+	unsigned long long irq_value;
+	unsigned long long value;
 	char *endptr;
 	int i;
 
@@ -182,7 +182,7 @@ static int irq_read (void)
 		for (i = 1; i < fields_num; i++)
 		{
 			errno = 0;
-			value = strtol (fields[i], &endptr, 10);
+			value = strtoull (fields[i], &endptr, 10);
 
 			if ((*endptr != '\0') || (errno != 0))
 				break;
@@ -190,8 +190,10 @@ static int irq_read (void)
 			irq_value += value;
 		} /* for (i) */
 
-		irq_submit (irq, irq_value);
+		/* Force 32bit wrap-around */
+		irq_submit (irq, irq_value % 4294967296ULL);
 	}
+
 	fclose (fh);
 
 	return (0);
